import json as json

from typing import Union as Union


# Json operate

def read_json(path: str, encoding: str = 'utf-8'):
    """Read json file."""

    with open(path, 'r', encoding=encoding) as f:
        return json.loads(f.read())


def save_json(path: str, data: Union[dict, list], encoding: str = 'utf-8'):
    """Save json file."""

    with open(path, 'w', encoding=encoding) as f:
        return f.write(json.dumps(data))


# List

def add_item_to_list(_list: list, item):
    """Add item to list if item not in list."""

    if item not in _list:
        _list.append(item)


def remove_list_item(_list: list, item):
    """Remove list item."""

    if item in _list:
        _list.remove(item)



import random
import re
import string

from typing import Union

from .check import isbytes, isstr


_RANDOM_LETTERS = string.ascii_letters + string.digits


def random_str(min_l: int = 8, max_l: int = 8):
    return ''.join(random.choice(_RANDOM_LETTERS) for i in range(random.randint(min_l, max_l)))


def s2b(text: str, encoding: str = 'utf-8') -> Union[bytes, None]:
    """Convert string to bytes."""

    try:
        if isstr(text):
            return bytes(text, encoding)
        if not isbytes(text):
            raise ValueError('Data is not string or bytes!')
        return text
    except:
        return None


def b2s(byte: bytes, encoding: str = 'utf-8') -> Union[str, None]:
    """Convert bytes to string."""

    try:
        if isbytes(byte):
            return bytes.decode(byte, encoding)
        if not isstr(byte):
            raise ValueError('Data is not bytes or string!')
        return byte
    except:
        return None


# Text

def search_text(
    pattern: re.Pattern,
    text: str,
    group_index: int = 0,
    **kwargs
):
    """Search text by passern and return result."""

    result = re.search(pattern, text, **kwargs)
    return result[group_index] if result else None



import socket
import urllib3


# Network utils

def get_domain_ip(domain: str):
    return socket.gethostbyname(domain)


def get_host(url: str, get_raw_data: bool = False):
    """Get the host of the input url."""

    host = urllib3.get_host(url)
    return host if get_raw_data else host[1]


def domains_is_ip(domains: list[str], ip: str):
    return all(
        [
            get_domain_ip(domain) == ip
        ] for domain in domains
    )


def int_ip2ip(int_ip: str):
    """Convert int ip to string."""

    if isinstance(int_ip, str):
        return int_ip

    ip = []

    for _ in range(4):
        ip.append(str(int_ip & 255))
        int_ip >>= 8

    return '.'.join(ip[::-1])


def ip2int_ip(ip: str):
    """Convert string ip to int."""

    if isinstance(ip, int):
        return ip

    int_ip = 0

    for i in ip.split('.'):
        int_ip = int_ip << 8 | int(i)

    return int_ip



import aiofiles
import io
import magic
import os
import shutil

from typing import Callable, Union


# File

async def async_read_file(path: str, **kwargs):
    """Async read file."""

    async with aiofiles.open(path, 'rb', **kwargs) as f:
        return await f.read()


async def async_save_file(
    path: str,
    file: Union[bytes, io.BytesIO, io.FileIO, str],
    replace: bool = True,
    **kwargs
):
    """Async save file."""

    mode = 'w' if isinstance(file, str) else 'wb'

    try:
        if os.path.exists(path) and not replace:
            raise FileExistsError()
        if getattr(file, 'read', None):
            file = file.read()

        async with aiofiles.open(path, mode, **kwargs) as f:
            await f.write(file)

        return True
    except:
        return False


def clear_dir(path: str):
    """Clear dir. (Remove and create.)"""

    rmdir(path)
    mkdirs(path)


def del_file(path: str):
    """Del file."""

    try:
        os.remove(path)
        return True
    except:
        return False


def get_file_mime(file: Union[bytes, io.BytesIO, io.FileIO]):
    """Get file mime."""

    is_file = getattr(file, 'read', None) != None
    data = file.read(2048) if is_file else file[:2048]
    file_mime = magic.from_buffer(data, mime=True)

    if is_file:
        file.seek(0)
    if file_mime:
        return file_mime.split('/')


def mkdir(path: str):
    """Create dir."""

    try:
        os.mkdir(path)
        return True
    except:
        return False


def mkdirs(path: str):
    """Create dir (use makedirs)."""

    try:
        os.makedirs(path)
        return True
    except:
        return False


def read_file(path: str):
    """Read file."""

    try:
        with open(path, 'rb') as f:
            data = f.read()
            return data
    except:
        pass


def rmdir(path: str):
    """Remove dir."""

    try:
        shutil.rmtree(path)
        return True
    except:
        return False


def save_file(
    path: str,
    file: Union[bytes, io.BytesIO, io.FileIO, str],
    replace: bool = True
):
    """Save file."""

    mode = 'w' if isinstance(file, str) else 'wb'

    try:
        if os.path.exists(path) and not replace:
            raise FileExistsError()
        if getattr(file, 'read', None):
            file = file.read()
        with open(path, mode) as f:
            f.write(file)
        return True
    except:
        return False


def save_file_as_bytesio(
    save_fnc: Callable,
    get_bytes: bool = False,
    **kwargs
):
    """Save file to io.BytesIO."""

    with io.BytesIO() as output:
        save_fnc(output, **kwargs)
        file_bytes = output.getvalue()

    if get_bytes:
        return file_bytes

    return io.BytesIO(file_bytes)


def move_file(path: str, target_path: str):
    """Move file or dir."""

    try:
        shutil.move(path, target_path)
        return True
    except:
        return False


def rename(path: str, name: str):
    """Rename file or dir."""

    try:
        os.rename(path, name)
        return True
    except:
        return False



import requests

from .network import get_host


_HEADERS = headers = {
    'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
    'accept-encoding': '*',
    'accept-language': 'zh-TW,zh;q=0.9,en-US;q=0.8,en;q=0.7',
    'cache-control': 'max-age=0',
    'Connection': 'keep-alive',
    'sec-ch-dpr': '1',
    'sec-ch-ua': '" Not A;Brand";v="99", "Chromium";v="103", "Google Chrome";v="103"',
    'sec-ch-ua-arch': '"x86"',
    'sec-ch-ua-bitness': '"64"',
    'sec-ch-ua-full-version': '"103.0.5060.114"',
    'sec-ch-ua-full-version-list': '".Not/A)Brand";v="99.0.0.0", "Google Chrome";v="103.0.5060.114", "Chromium";v="103.0.5060.114"',
    'sec-ch-ua-mobile': '?0',
    'sec-ch-ua-model': '""',
    'sec-ch-ua-platform-version': '"14.0.0"',
    'sec-ch-ua-wow64': '?0',
    'sec-fetch-dest': 'document',
    'sec-fetch-mode': 'navigate',
    'sec-fetch-site': 'same-origin',
    'sec-fetch-user': '?1',
    'upgrade-insecure-requests': '1',
    'user-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36'
}


# Response and requests

def get_request_headers(
    url: str,
    add_origin: bool = True,
    referer: str = None,
    **kwargs
):
    """Get requests headers."""

    host = get_host(url)

    headers = {
        **_HEADERS,
        **kwargs
    }

    if add_origin:
        headers['origin'] = host

    if referer:
        headers['referer'] = referer

    return headers


def get_response(
    url: str,
    method: str = 'POST',
    method_data: dict = {},
    cookies: dict = {},
    files: dict = {},
    headers: bool = True,
    extra_headers: dict = {},
    header_add_host: bool = True,
    header_referer: str = None,
    proxies: dict = {},
    max_redirect: int = 5,
    timeout: int = 3,
    verify: bool = True
):
    """Get the request."""

    max_redirect += 1
    redirect_urls = set()

    while max_redirect:
        try:
            kwargs = {
                'url': url,
                'method': method,
                'cookies': cookies,
                'files': files,
                'proxies': proxies,
                'timeout': timeout,
                'allow_redirects': False,
                'verify': verify
            }

            if method.lower() == 'get':
                kwargs['params'] = method_data
            elif method.lower() == 'post':
                kwargs['data'] = method_data

            if headers:
                headers = get_request_headers(
                    url,
                    header_add_host,
                    header_referer
                )

                kwargs['headers'] = {
                    **headers,
                    **extra_headers
                }

            response = requests.request(**kwargs)

            if 300 <= response.status_code <= 310:
                redirect_to = response.headers['location']

                if redirect_to in redirect_urls:
                    break

                url = redirect_to
                redirect_urls.add(url)
                max_redirect -= 1
            else:
                break
        except:
            return None

    return response



import datetime as datetime
import time as time


# Time

def get_time_zone_offset(get_type: str = 's'):
    """Get time zone offset. Default return seconds."""

    zone_offset = time.timezone if (time.localtime().tm_isdst == 0) else time.altzone

    if get_type == 'h':
        return round(zone_offset / 3600)
    else:
        return zone_offset


def int_time(time_str: str, format_str: str = '%Y-%m-%d %a %H:%M:%S', use_zone: bool = False):
    """Convert string datetime to timestamp."""

    struct_time = time.strptime(time_str, format_str)
    timestamp = int(time.mktime(struct_time))

    if use_zone:
        return timestamp + get_time_zone_offset()
    else:
        return timestamp


def now_time(get_timestamp: bool = False, str_format: str = '%Y-%m-%d %a %H:%M:%S'):
    """Get now time."""

    now = datetime.datetime.now()
    return int(time.mktime(now.timetuple())) if get_timestamp else str(now.strftime(str_format))


def now_time_ms():
    """Get now timestamp(ms)."""

    return round(time.time() * 1000)


def now_time_utc():
    """Get now utc timestamp."""

    zone_offset = get_time_zone_offset()
    return now_time(True) + zone_offset


def now_time_utc_ms():
    """Get now utc timestamp(ms)."""

    zone_offset = get_time_zone_offset()
    return now_time_ms() + (zone_offset * 1000)


def str_time(
    timestamp: int,
    str_format: str = '%Y-%m-%d %a %H:%M:%S',
    use_zone: bool = False
):
    """Change timestamp to str time."""

    if use_zone:
        timestamp -= get_time_zone_offset()

    date_time = datetime.datetime.fromtimestamp(timestamp)
    return date_time.strftime(str_format)



from typing import Union as Union

from .check import isint_or_digit


# Number

def get_int_data(data: Union[str, int], default=None):
    """Input int or string, if input value is not int or isdigit, return default value."""

    return int(data) if isint_or_digit(data) else default



import json

from binascii import a2b_hex, b2a_hex
from Cryptodome.Cipher import AES
from typing import Union

from .check import isdict, islist
from .string import b2s, s2b
from .hash import md5


class AesCrypt:
    def __init__(
        self,
        key: Union[bytes, str],
        iv: Union[bytes, str] = None,
        mode=AES.MODE_CBC
    ):
        iv = s2b(iv)
        key = md5(key, True)

        if mode == AES.MODE_ECB:
            self.cipher = AES.new(key, mode)
        else:
            self.cipher = AES.new(key, mode, iv=iv)

    def pad(self, text: Union[dict, list, str]):
        if isdict(text) or islist(text):
            text = json.dumps(text)

        text = s2b(text)

        while len(text) % 16 != 0:
            text += b' '

        return text

    def encrypt(self, text: Union[dict, list, str]):
        text = self.pad(text)
        ciphertext = self.cipher.encrypt(text)
        return b2s(b2a_hex(ciphertext))

    def decrypt(self, ciphertext: str):
        ciphertext = a2b_hex(s2b(ciphertext))
        text = self.cipher.decrypt(ciphertext)

        try:
            text = b2s(text).rstrip()

            try:
                data = json.loads(text)
                return data
            except:
                return text
        except:
            return text.rstrip()



import uuid

from .file import read_file, save_file
from .string import b2s


# UUID

def get_uuid(save_path: str = './uuid.uuid'):
    if now_uuid := read_file(save_path):
        return b2s(now_uuid)

    now_uuid = uuid.uuid1()
    now_uuid = str(uuid.uuid1())
    save_file(save_path, now_uuid)
    return now_uuid



import hashlib

from typing import Callable

from .string import s2b


def hash(fnc: Callable, text: str | bytes, return_bytes: bool) -> str | bytes:
    return fnc(s2b(text)).digest() if return_bytes else fnc(s2b(text)).hexdigest()


def md5(text: str | bytes, return_bytes: bool = False):
    return hash(hashlib.md5, text, return_bytes)


def sha3_224(text: str | bytes, return_bytes: bool = False):
    return hash(hashlib.sha3_224, text, return_bytes)


def sha3_256(text: str | bytes, return_bytes: bool = False):
    return hash(hashlib.sha3_256, text, return_bytes)


def sha3_384(text: str | bytes, return_bytes: bool = False):
    return hash(hashlib.sha3_384, text, return_bytes)


def sha3_512(text: str | bytes, return_bytes: bool = False):
    return hash(hashlib.sha3_512, text, return_bytes)



from setuptools import find_packages, setup


setup(
    name='kiki_utils',
    classifiers=[
        'License :: Freely Distributable'
    ],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=True,
    version='1.1.0',
    description='Utils functions without cv2 and pillow',
    author='kiki-kanri',
    author_email='a470666@gmail.com',
    keywords=['Utils'],
    install_requires=[
        'aiofiles',
        'loguru',
        'pycryptodomex',
        'pyopenssl',
        'python-magic;platform_system=="Linux"',
        'python-magic-bin;platform_system=="Windows"',
        'requests'
    ],
    python_requires=">=3.6"
)



import sys

from loguru import logger


logger.remove()


def add_logger_path(add_logger, path, colorize: bool = False):
    add_logger.add(
        path,
        format='<level>{time:YYYY-MM-DD HH:mm:ss}｜{level}｜{message}</level>',
        colorize=colorize
    )


add_logger_path(logger, sys.stdout, True)



import os
import re

from typing import Union


ALLOWED_EMAILS = [
    'gmail.com',
    'yahoo.com',
    'hotmail.com',
    'aol.com',
    'hotmail.co.uk',
    'hotmail.fr',
    'msn.com',
    'wanadoo.fr',
    'live.com',
    'hotmail.it',
    'qq.com'
]


# Check

def check_domain(domain: str):
    """Check domain ping."""

    domain = re.sub(r'[;|&\-\s​]', '', domain)
    return os.system(f'ping -c 1 -s 8 {domain}') == 0


def check_email(email: str):
    """Check email format and ping the domain."""

    if re.match(r'.*[+\-*/\\;&|\s​].*', email):
        return False

    domain = email.split('@')[-1].lower()

    if domain.lower() in ALLOWED_EMAILS:
        return True

    return check_domain(domain)


def isint_or_digit(text: Union[int, str]):
    """Check if the value is int or isdigit."""

    return isint(text) or (isstr(text) and text.isdigit())


def isbytes(*args):
    """Determine whether it is bytes."""

    return all([isinstance(arg, bytes) for arg in args])


def isdict(*args):
    """Determine whether it is dict."""

    return all([isinstance(arg, dict) for arg in args])


def isfile(*args):
    """Determine whether it is file."""

    return all([os.path.isfile(arg) for arg in args])


def isint(*args):
    """Determine whether it is int."""

    return all([isinstance(arg, int) for arg in args])


def islist(*args):
    """Determine whether it is list."""

    return all([isinstance(arg, list) for arg in args])


def isstr(*args):
    """Determine whether it is str."""

    return all([isinstance(arg, str) for arg in args])
